@@ -46,7 +46,8 @@ public enum GuardExceptionCodes implements DisplayableOXExceptionCode {
     URI_PARSE_FAILED("Unable to parse IMAP server URI \"%1$s\".",GuardExceptionMessages.URI_PARSE_FAILED_MSG, CATEGORY_ERROR,6),
     ACTION_NOT_ALLOWED("Action not allowed. '%1$s'", GuardExceptionMessages.ACTION_NOT_ALLOWED_MSG, CATEGORY_ERROR, 7),
     ACTION_NOT_SUPPORTED("Action not allowed. '%1$s'", GuardExceptionMessages.ACTION_NOT_SUPPORTED_MSG, CATEGORY_ERROR, 8),
-    BLOCKED_BY_PIN("Unable to perform action until pin is removed", GuardExceptionMessages.BLOCKED_BY_PIN, CATEGORY_ERROR, 9)
+    BLOCKED_BY_PIN("Unable to perform action until pin is removed", GuardExceptionMessages.BLOCKED_BY_PIN, CATEGORY_ERROR, 9),
+    MISSING_KEYS("No keys found", GuardExceptionMessages.MISSING_KEYS, CATEGORY_ERROR, 10),
     ;
 
     private final String message;
