@@ -49,6 +49,6 @@ public class GuardAwareLoginHandler implements LoginHandlerService {
      */
     @Override
     public void handleLogout(LoginResult logout) throws OXException {
-        new AuthenticationTokenHandler().destroyForSession(logout.getSession(), true);
+        new AuthenticationTokenHandler().destroyForSession(logout.getSession(), true, null);
     }
 }
