@@ -21,8 +21,11 @@
 
 package com.openexchange.guard.interceptor;
 
+import java.util.List;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
 import com.openexchange.guard.internal.authentication.AuthenticationTokenValidator;
@@ -40,39 +43,33 @@ import com.openexchange.session.inspector.SessionInspectorService;
  */
 public class GuardAwareSessionInspector implements SessionInspectorService {
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see com.openexchange.session.inspector.SessionInspectorService#onSessionHit(com.openexchange.session.Session, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
-     */
+    /** Simple class to delay initialization until needed */
+    private static class LoggerHolder {
+
+        static final Logger LOGGER = LoggerFactory.getLogger(GuardAwareSessionInspector.class);
+    }
+
     @Override
     public Reply onSessionHit(Session session, HttpServletRequest request, HttpServletResponse response) throws OXException {
         AuthenticationTokenHandler tokenHandler = new AuthenticationTokenHandler();
-        GuardAuthenticationToken authToken = tokenHandler.getForSession(session, false /* do not consume the token here */);
-        if (authToken != null) {
-            if (!new AuthenticationTokenValidator().isValid(authToken)) {
-                //Just remove the token from the session. The token is still valid on the Guard server since a client could still remember it.
-                tokenHandler.removeForSession(session);
+        List<GuardAuthenticationToken> tokens = tokenHandler.getAllForSession(session);
+        tokens.forEach((t) -> {
+            if (!AuthenticationTokenValidator.isValid(t)) {
+                try {
+                    tokenHandler.removeForSession(session, t.getType().getValue());
+                } catch (OXException e) {
+                    LoggerHolder.LOGGER.debug("Unable to remove session by tokens", e);
+                }
             }
-        }
+        });
         return Reply.NEUTRAL;
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see com.openexchange.session.inspector.SessionInspectorService#onSessionMiss(java.lang.String, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
-     */
     @Override
     public Reply onSessionMiss(String sessionId, HttpServletRequest request, HttpServletResponse response) throws OXException {
         return Reply.NEUTRAL;
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see com.openexchange.session.inspector.SessionInspectorService#onAutoLoginFailed(com.openexchange.session.inspector.Reason, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
-     */
     @Override
     public Reply onAutoLoginFailed(Reason reason, HttpServletRequest request, HttpServletResponse response) throws OXException {
         return Reply.NEUTRAL;
