@@ -36,10 +36,10 @@ import com.openexchange.guard.exceptions.GuardExceptionCodes;
 public abstract class AbstractGuardAccess {
 
     public static final String GUARDADMIN_ENDPOINT = "guardadmin";
-    public static final String PGP_MAIL_ENDPOINT = "oxguard/pgpmail";
     public static final String PGPCORE_ENDPOINT = "oxguard/pgpcore";
     public static final String LOGIN_ENDPOINT = "oxguard/login";
     public static final String GUEST_ENDPOINT = "oxguard/guest";
+    public static final String CRYPTO_ENDPOINT = "oxguard/crypto";
 
     private static final AtomicReference<Map<String, GuardApiImpl>> API_REF = new AtomicReference<Map<String, GuardApiImpl>>();
 
