@@ -96,6 +96,7 @@ import com.openexchange.guard.api.GuardApiExceptionCodes;
 import com.openexchange.guard.internal.authentication.AuthenticationTokenUtils;
 import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
 import com.openexchange.guard.osgi.GuardHttpClientConnetionConfiguration;
+import com.openexchange.guard.osgi.Services;
 import com.openexchange.java.Charsets;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
@@ -113,6 +114,7 @@ import com.openexchange.rest.client.httpclient.HttpClientService;
 import com.openexchange.rest.client.httpclient.ManagedHttpClient;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.session.Session;
+import com.openexchange.user.UserService;
 
 /**
  * {@link GuardApiImpl}
@@ -125,6 +127,10 @@ public class GuardApiImpl implements GuardApi {
 
     private static final Logger LOGGER = org.slf4j.LoggerFactory.getLogger(GuardApiImpl.class);
 
+    public static Logger getLogger() {
+        return LOGGER;
+    }
+
     // -------------------------------------------------------------------------------------------------------------- //
 
     /** The status code policy to obey */
@@ -151,51 +157,46 @@ public class GuardApiImpl implements GuardApi {
                 if (404 == statusCode) {
                     throw GuardApiExceptionCodes.NOT_FOUND_SIMPLE.create();
                 }
-                String reason;
+                String reason = statusLine.getReasonPhrase();
                 String code;
+                HttpResponseException exception = null;
                 try {
                     JSONObject jsonObject = null;
                     String jsonBody = getJsonString(httpResponse);
                     if (jsonBody == null) {
                         jsonBody = "";  // No reason for error
                     }
-                    if (jsonBody.trim().startsWith("[")) {
-                        JSONArray jsonArray = new JSONArray(jsonBody);
-                        if (jsonArray.length() > 0) {
-                            jsonObject = jsonArray.getJSONObject(0);
-                            code = jsonObject.getString("code");
-                            reason = code + ": " + jsonObject.getString("error_desc");
-                        } else {
-                            reason = statusLine.getReasonPhrase();
-                        }
-                    } else {
-                        //Old style of error response? remove this?
-                        if (jsonBody.startsWith("{")) {
-                            jsonObject = new JSONObject(jsonBody);
+                    if (jsonBody.startsWith("{")) {
+                        jsonObject = new JSONObject(jsonBody);
+                        if (jsonObject.has("code") && jsonObject.has("error_desc")) {
                             code = jsonObject.getString("code");
                             reason = code + ": " + jsonObject.getString("error_desc");
+                            exception = new ApiResponseException(statusCode, reason, jsonObject);
                         } else {
                             reason = jsonBody;
                         }
                     }
-
-                } catch (final Exception e) {
+                } catch (final IOException | JSONException e) {
+                    getLogger().error("Problem parsing error response from Guard MW", e);
                     reason = statusLine.getReasonPhrase();
                 }
-                throw new HttpResponseException(statusCode, reason);
+                if (exception == null) {
+                    exception = new HttpResponseException(statusCode, reason);
+                }
+                throw exception;
             }
         }
     };
 
     /**
      * Parse out JSON from HTTP response
-     * 
+     *
      * @param httpResponse
      * @return
      * @throws ParseException
      * @throws IOException
      */
-    private static String getJsonString(HttpResponse httpResponse) throws ParseException, IOException {
+    static String getJsonString(HttpResponse httpResponse) throws ParseException, IOException {
         String jsonBody = EntityUtils.toString(httpResponse.getEntity(), Charsets.UTF_8);
         if (jsonBody != null && jsonBody.contains("{") && jsonBody.contains("}")) {
             jsonBody = jsonBody.substring(jsonBody.indexOf("{"));
@@ -212,28 +213,33 @@ public class GuardApiImpl implements GuardApi {
             final StatusLine statusLine = httpResponse.getStatusLine();
             final int statusCode = statusLine.getStatusCode();
             if ((statusCode < 200 || statusCode >= 300) && statusCode != 404) {
-                String reason;
+                String reason = statusLine.getReasonPhrase();
+                String code;
+                HttpResponseException exception = null;
                 try {
                     JSONObject jsonObject = null;
-                    String jsonBody = EntityUtils.toString(httpResponse.getEntity(), Charsets.UTF_8);
-                    if (jsonBody != null && jsonBody.trim().startsWith("[")) {
-                        JSONArray jsonArray = new JSONArray(jsonBody);
-                        if (jsonArray.length() > 0) {
-                            jsonObject = jsonArray.getJSONObject(0);
-                            reason = jsonObject.getString("error_desc");
-                        } else {
-                            reason = statusLine.getReasonPhrase();
-                        }
-                    } else {
-                        //Old style of error response? remove this?
-                        jsonObject = new JSONObject(new InputStreamReader(httpResponse.getEntity().getContent(), Charsets.UTF_8));
-                        reason = jsonObject.getJSONObject("error").getString("message");
+                    String jsonBody = getJsonString(httpResponse);
+                    if (jsonBody == null) {
+                        jsonBody = "";  // No reason for error
                     }
-
-                } catch (final Exception e) {
+                    if (jsonBody.startsWith("{")) {
+                        jsonObject = new JSONObject(jsonBody);
+                        if (jsonObject.has("code") && jsonObject.has("error_desc")) {
+                            code = jsonObject.getString("code");
+                            reason = code + ": " + jsonObject.getString("error_desc");
+                            exception = new ApiResponseException(statusCode, reason, jsonObject);
+                        } else {
+                            reason = jsonBody;
+                        }
+                    }
+                } catch (final IOException | JSONException e) {
+                    getLogger().error("Problem parsing error response from Guard MW", e);
                     reason = statusLine.getReasonPhrase();
                 }
-                throw new HttpResponseException(statusCode, reason);
+                if (exception == null) {
+                    exception = new HttpResponseException(statusCode, reason);
+                }
+                throw exception;
             }
         }
     };
@@ -316,7 +322,7 @@ public class GuardApiImpl implements GuardApi {
     /**
      * Pull IP address from session.
      * If localhost, try grizzly remote address from Log
-     * 
+     *
      * @param session
      * @return
      */
@@ -359,6 +365,7 @@ public class GuardApiImpl implements GuardApi {
         InputStream msgSrc = null;
         try {
             request = new HttpPost(buildUri(toQueryString(parameters)));
+            setLanguage(request, session);
             final InputStream data = MimeMessageUtility.getStreamFromPart(mimeMessage);
 
             final String boundary = UUIDs.getUnformattedStringFromRandom();
@@ -437,7 +444,7 @@ public class GuardApiImpl implements GuardApi {
 
     /**
      * Find any headers with Remove-recip. If found, remove the recipient specified from the address list
-     * 
+     *
      * @param tmp
      * @param recipients
      * @return
@@ -474,8 +481,9 @@ public class GuardApiImpl implements GuardApi {
         try {
             final URI uri = buildUri(toQueryString(parameters));
             request = new HttpGet(uri);
-
+            setLanguage(request, session);
             LOGGER.debug("Executing GET using \"{}\"", uri);
+
             return handleHttpResponse(execute(request, getHttpClient(), session), clazz);
         } catch (final HttpResponseException e) {
             if (400 == e.getStatusCode() || 401 == e.getStatusCode()) {
@@ -501,7 +509,7 @@ public class GuardApiImpl implements GuardApi {
 
             List<NameValuePair> urlEncodedParameters = toQueryString(bodyParameters);
             request.setEntity(new UrlEncodedFormEntity(urlEncodedParameters));
-
+            setLanguage(request, session);
             LOGGER.debug("Executing POST using \"{}\"", uri);
             return handleHttpResponse(execute(request, getHttpClient(), session), clazz);
         } catch (final HttpResponseException e) {
@@ -526,7 +534,7 @@ public class GuardApiImpl implements GuardApi {
             final URI uri = buildUri(toQueryString(parameters));
             request = new HttpPut(uri);
             request.setEntity(asHttpEntity(jsonBody));
-
+            setLanguage(request, session);
             LOGGER.debug("Executing PUT using \"{}\" with body \"{}\"", uri, jsonBody);
             return handleHttpResponse(execute(request, getHttpClient(), session), clazz);
         } catch (final HttpResponseException e) {
@@ -552,7 +560,7 @@ public class GuardApiImpl implements GuardApi {
         try {
             final URI uri = buildUri(toQueryString(parameters));
             request = new HttpGet(uri);
-
+            setLanguage(request, session);
             LOGGER.debug("Executing GET using \"{}\"", uri);
             final InputStream data = handleHttpResponse(execute(request, getHttpClient(), session), InputStream.class);
             final ResourceReleasingInputStream in = new ResourceReleasingInputStream(data, request);
@@ -586,7 +594,7 @@ public class GuardApiImpl implements GuardApi {
         InputStream processedResource = null;
         try {
             request = new HttpPost(buildUri(toQueryString(parameters)));
-
+            setLanguage(request, session);
             final String boundary = UUIDs.getUnformattedStringFromRandom();
             MultipartEntity multipartEntity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE, boundary, Charsets.UTF_8) {
 
@@ -750,7 +758,6 @@ public class GuardApiImpl implements GuardApi {
                 request.setConfig(RequestConfig.custom().setCookieSpec(CookieSpecs.IGNORE_COOKIES).build());
                 request.setHeader("Cookie", sCookies.toString());
             }
-
             final BasicHttpContext context = new BasicHttpContext();
             final BasicScheme basicAuth = new BasicScheme();
             context.setAttribute("preemptive-auth", basicAuth);
@@ -824,7 +831,6 @@ public class GuardApiImpl implements GuardApi {
                 request.setConfig(RequestConfig.custom().setCookieSpec(CookieSpecs.IGNORE_COOKIES).build());
                 request.setHeader("Cookie", sCookies.toString());
             }
-
             final BasicHttpContext context = new BasicHttpContext();
             final BasicScheme basicAuth = new BasicScheme();
             context.setAttribute("preemptive-auth", basicAuth);
@@ -901,7 +907,6 @@ public class GuardApiImpl implements GuardApi {
                 request.setConfig(RequestConfig.custom().setCookieSpec(CookieSpecs.IGNORE_COOKIES).build());
                 request.setHeader("Cookie", sCookies.toString());
             }
-
             final BasicHttpContext context = new BasicHttpContext();
             final BasicScheme basicAuth = new BasicScheme();
             context.setAttribute("preemptive-auth", basicAuth);
@@ -976,7 +981,6 @@ public class GuardApiImpl implements GuardApi {
                 request.setConfig(RequestConfig.custom().setCookieSpec(CookieSpecs.IGNORE_COOKIES).build());
                 request.setHeader("Cookie", sCookies.toString());
             }
-
             final BasicHttpContext context = new BasicHttpContext();
             final BasicScheme basicAuth = new BasicScheme();
             context.setAttribute("preemptive-auth", basicAuth);
@@ -1262,6 +1266,9 @@ public class GuardApiImpl implements GuardApi {
         if (e.getMessage() == null || e.getMessage().isEmpty()) {
             return GuardApiExceptionCodes.GUARD_SERVER_ERROR.create(e, Integer.valueOf(e.getStatusCode()), "No error message provided");
         }
+        if (e instanceof ApiResponseException) {
+            return new ApiDynamicExceptionCodes(((ApiResponseException) e).getJson(), e.getMessage()).create();
+        }
         return GuardApiExceptionCodes.GUARD_SERVER_ERROR.create(e, Integer.valueOf(e.getStatusCode()), e.getMessage());
     }
 
@@ -1285,6 +1292,50 @@ public class GuardApiImpl implements GuardApi {
         return request;
     }
 
+    /**
+     * Add locale cookie to request
+     *
+     * @param request
+     * @param session
+     */
+    private static void setLanguage(HttpRequestBase request, Session session) {
+        if (session == null) {
+            return;
+        }
+        UserService userService = Services.getService(UserService.class);
+        try {
+            String locale = userService.getUser(session.getUserId(), session.getContextId()).getLocale().toString();
+            setLanguage(request, locale);
+        } catch (OXException e) {
+            LOGGER.error("Problem setting language for api call", e);
+        }
+    }
+
+    /**
+     * Adds locale cookie to request
+     *
+     * @param request
+     * @param language
+     */
+    private static void setLanguage(HttpRequestBase request, String language) {
+        if (Strings.isEmpty(language)) {
+            return;
+        }
+        Header cookies = request.getFirstHeader("Cookie");
+        StringBuilder sCookies = new StringBuilder();
+        if (cookies != null) {
+            String existing = cookies.getValue();
+            if (Strings.isNotEmpty(existing) && existing.contains("locale")) {
+                return;
+            }
+            sCookies.append(existing);
+            sCookies.append(";");
+        }
+        Cookie cookie = new Cookie("locale", language);
+        sCookies.append(cookie.getName()).append('=').append(cookie.getValue());
+        request.setHeader("Cookie", sCookies.toString());
+    }
+
     /**
      * An <code>InputStream</code> that takes care of shutting-down associated <code>HttpClient</code> instance when closed.
      */
