@@ -21,15 +21,20 @@
 
 package com.openexchange.guard.internal.authentication;
 
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.List;
 import java.util.concurrent.locks.Lock;
 import org.json.JSONException;
 import org.json.JSONObject;
+import com.openexchange.crypto.CryptoType.PROTOCOL;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.api.GuardApis;
 import com.openexchange.guard.exceptions.GuardExceptionCodes;
 import com.openexchange.guard.internal.AbstractGuardAccess;
 import com.openexchange.guard.internal.GuardApiImpl;
 import com.openexchange.guard.osgi.Services;
+import com.openexchange.java.Strings;
 import com.openexchange.session.Session;
 import com.openexchange.sessiond.SessiondService;
 
@@ -41,7 +46,7 @@ import com.openexchange.sessiond.SessiondService;
  */
 public class AuthenticationTokenHandler extends AbstractGuardAccess {
 
-    private static org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(AuthenticationTokenHandler.class);
+    private static final org.slf4j.Logger LOGGER = org.slf4j.LoggerFactory.getLogger(AuthenticationTokenHandler.class);
     private static final String AUTH_TOKEN_PARAMETER = "GUARD_AUTH_TOKEN";
 
     /**
@@ -59,7 +64,7 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
             json.putSafe("session", authToken.getGuardSessionId());
             guardApi.doCallPut(GuardApis.mapFor("action", "deleteauthtoken"), json, Void.class, null);
         } else {
-            logger.error("Endpoint not available " + PGPCORE_ENDPOINT);
+            LOGGER.error("Endpoint not available " + PGPCORE_ENDPOINT);
         }
     }
 
@@ -68,18 +73,24 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
      *
      * @param session The session to get the authentication token for
      * @param markAsUsed Whether to mark the token as used or not
+     * @param type The crypto type
      * @return An authentication token for the given session, or null if no authentication token was attached to the session
-     * @throws OXException
+     * @throws OXException In case of error
      */
-    public GuardAuthenticationToken getForSession(Session session, boolean markAsUsed) throws OXException {
+    public GuardAuthenticationToken getForSession(Session session, boolean markAsUsed, String type) throws OXException {
+        if (Strings.isEmpty(type)) {
+            type = "PGP";
+        }
         final Lock lock = (Lock) session.getParameter(Session.PARAM_LOCK);
         lock.lock();
         try {
-            if (session.containsParameter(AUTH_TOKEN_PARAMETER)) {
-                GuardAuthenticationToken authToken = GuardAuthenticationToken.fromString((String) session.getParameter(AUTH_TOKEN_PARAMETER));
+            SessionAuthToken tokens = new SessionAuthToken(session.getParameter(AUTH_TOKEN_PARAMETER));
+            String token = tokens.getAuthToken(type);
+            if (token != null) {
+                GuardAuthenticationToken authToken = GuardAuthenticationToken.fromString(token);
                 if (markAsUsed && authToken != null && authToken.getMinutesValid() < 0) {
                     authToken.setUsed();
-                    setForSession(session, authToken);
+                    setForSession(session, authToken, type);
                 }
                 return authToken;
             }
@@ -89,15 +100,40 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
         }
     }
 
+    /**
+     * Get all authentication tokens for session
+     *
+     * @param session The session to get the authentication token for
+     * @return List of authTokens
+     */
+    public List<GuardAuthenticationToken> getAllForSession(Session session) {
+        final Lock lock = (Lock) session.getParameter(Session.PARAM_LOCK);
+        lock.lock();
+        ArrayList<GuardAuthenticationToken> tokens = new ArrayList<GuardAuthenticationToken>();
+        try {
+            SessionAuthToken sessionTokens = new SessionAuthToken(session.getParameter(AUTH_TOKEN_PARAMETER));
+            for (String token : sessionTokens.getAllTokens()) {
+                if (token != null) {
+                    GuardAuthenticationToken authToken = GuardAuthenticationToken.fromString(token);
+                    tokens.add(authToken);
+                }
+            }
+            return tokens;
+        } finally {
+            lock.unlock();
+        }
+    }
+
     /**
      * Gets the authentication token for a given session and mark the token as "used"
      *
      * @param session The session to get he authentication token for
      * @return The authentication token for the given session, or null if no authentication token was attached to the session
-     * @throws OXException
+     * @param type The crypto type
+     * @throws OXException In case of error
      */
-    public GuardAuthenticationToken getForSession(Session session) throws OXException {
-        return getForSession(session, true);
+    public GuardAuthenticationToken getForSession(Session session, String type) throws OXException {
+        return getForSession(session, true, type);
     }
 
     /**
@@ -105,10 +141,11 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
      *
      * @param session The session to get the authentication token for
      * @return The authentication token for the given session
+     * @param type The crypto type
      * @throws OXException If no such token was found for the given session
      */
-    public GuardAuthenticationToken requireForSession(Session session) throws OXException {
-        GuardAuthenticationToken authToken = getForSession(session, true);
+    public GuardAuthenticationToken requireForSession(Session session, String type) throws OXException {
+        GuardAuthenticationToken authToken = getForSession(session, true, type);
         if (authToken != null) {
             return authToken;
         }
@@ -120,14 +157,16 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
      *
      * @param session The session
      * @param authToken The authentication token
-     * @return
-     * @throws OXException
+     * @param type The crypto type
+     * @throws OXException In case of error
      */
-    public void setForSession(Session session, GuardAuthenticationToken authToken) throws OXException {
+    public void setForSession(Session session, GuardAuthenticationToken authToken, String type) throws OXException {
         final Lock lock = (Lock) session.getParameter(Session.PARAM_LOCK);
         lock.lock();
         try {
-            session.setParameter(AUTH_TOKEN_PARAMETER, authToken.toString());
+            SessionAuthToken token = new SessionAuthToken(session.getParameter(AUTH_TOKEN_PARAMETER));
+            token.replaceAuthToken(authToken.toString(), type);
+            session.setParameter(AUTH_TOKEN_PARAMETER, token.toString());
         } finally {
             lock.unlock();
         }
@@ -142,29 +181,37 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
      *
      * @param session The session
      * @param destroy true, in order to invalidate the authentication on the Guard Server, false to just detach it from the session.
-     * @param logout If system logout.  If session being removed the hazelcast won't be notified
+     * @param logout If system logout. If session being removed the hazelcast won't be notified
      * @throws OXException
      */
-    private void removeForSessionInternal(Session session, boolean destroy, boolean logout) throws OXException {
-        GuardAuthenticationToken authToken = null;
-
+    private void removeForSessionInternal(Session session, boolean destroy, boolean logout, String type) throws OXException {
+        List<GuardAuthenticationToken> authTokens = null;
         final Lock lock = (Lock) session.getParameter(Session.PARAM_LOCK);
         lock.lock();
         try {
-            authToken = getForSession(session, false);
-            if (authToken != null) {
-                session.setParameter(AUTH_TOKEN_PARAMETER, null);
+            // None specified, get all
+            if (PROTOCOL.NONE.getValue().equalsIgnoreCase(type) || Strings.isEmpty(type)) {
+                authTokens = getAllForSession(session);
+            } else {
+                authTokens = Collections.singletonList(getForSession(session, false, type));
             }
+            SessionAuthToken tokens = new SessionAuthToken(session.getParameter(AUTH_TOKEN_PARAMETER));
+            tokens.removeAuthToken(type);
+            session.setParameter(AUTH_TOKEN_PARAMETER, tokens.toString());
         } finally {
             lock.unlock();
         }
 
-        if (destroy && authToken != null) {
-            destroyAuthToken(authToken);
+        // If destroy, remove each
+        if (destroy && authTokens != null && authTokens.size() > 0) {
+            for (GuardAuthenticationToken token : authTokens) {
+                if (token != null)
+                    destroyAuthToken(token);
+            }
         }
 
         if (!logout) {
-            if (authToken != null && authToken.getMinutesValid() >= 0) {  // don't push for single use
+            if (authTokens != null && authTokens.size() > 0 && authTokens.get(0) != null && authTokens.get(0).getMinutesValid() >= 0) {  // don't push for single use
                 SessiondService sessionService = Services.getService(SessiondService.class);
                 sessionService.storeSession(session.getSessionID(), false);
             }
@@ -177,12 +224,13 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
      * This will NOT destroy / invalidate an authentication token on the Guard server.
      *
      * @param session The session
-     * @throws OXException
+     * @param type The crypto type
+     * @throws OXException In case of error
      */
-    public void removeForSession(Session session) throws OXException {
+    public void removeForSession(Session session, String type) throws OXException {
         final boolean destroy = false;
         final boolean logout = false;
-        removeForSessionInternal(session, destroy, logout);
+        removeForSessionInternal(session, destroy, logout, type);
     }
 
     /**
@@ -192,10 +240,11 @@ public class AuthenticationTokenHandler extends AbstractGuardAccess {
      *
      *
      * @param session The session
-     * @throws OXException
+     * @param type The crypto type
+     * @throws OXException In case of error
      */
-    public void destroyForSession(Session session, boolean logout) throws OXException {
+    public void destroyForSession(Session session, boolean logout, String type) throws OXException {
         final boolean destroy = true;
-        removeForSessionInternal(session, destroy, logout);
+        removeForSessionInternal(session, destroy, logout, type);
     }
 }
