@@ -21,8 +21,8 @@
 
 package com.openexchange.guard.internal.authentication;
 
-import java.util.Date;
 import static java.util.concurrent.TimeUnit.MINUTES;
+import java.util.Date;
 
 /**
  * {@link AuthenticationTokenValidator} validates an {@link GuardAuthenticationToken}
@@ -38,7 +38,7 @@ public class AuthenticationTokenValidator {
      * @param authToken The token to verify
      * @return True, if the token is still valid, false otherwise
      */
-    public boolean isValid(GuardAuthenticationToken authToken) {
+    public static boolean isValid(GuardAuthenticationToken authToken) {
         if (authToken != null) {
             if (authToken.getMinutesValid() > -1) {
 
