@@ -62,7 +62,7 @@ public abstract class AbstractGuardAction extends AbstractGuardAccess implements
          * Check module permissions
          */
         CapabilityService capabilityService = services.getService(CapabilityService.class);
-        if (false == capabilityService.getCapabilities(session).contains("guard")) {
+        if (false == (capabilityService.getCapabilities(session).contains("guard") || capabilityService.getCapabilities(session).contains("smime"))) {
             throw AjaxExceptionCodes.NO_PERMISSION_FOR_MODULE.create("guard");
         }
 
