@@ -21,14 +21,21 @@
 
 package com.openexchange.guard.json.actions;
 
+import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.java.Autoboxing.i;
 import java.nio.charset.StandardCharsets;
 import java.security.MessageDigest;
-import javax.servlet.http.Cookie;
+import java.util.Collections;
+import java.util.List;
 import javax.servlet.http.HttpServletRequest;
+import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
+import com.openexchange.crypto.CryptoType;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.api.GuardApis;
 import com.openexchange.guard.exceptions.GuardExceptionCodes;
@@ -38,8 +45,6 @@ import com.openexchange.guard.internal.authentication.GuardLoginHandler;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.tools.servlet.AjaxExceptionCodes;
 import com.openexchange.tools.session.ServerSession;
-import static com.openexchange.java.Autoboxing.i;
-import static com.openexchange.java.Autoboxing.I;
 
 /**
  * {@link AuthTokenAction} creates, attaches or detaches an {@link GuardAuthenticationToken} to or from a groupware session
@@ -49,13 +54,18 @@ import static com.openexchange.java.Autoboxing.I;
  */
 public class AuthTokenAction extends AbstractGuardAction {
 
+    private static final Logger LOGGER = LoggerFactory.getLogger(AuthTokenAction.class);
+
+
     private static final String FIELD_MINUTES_VALID = "minutesValid";
     private static final String FIELD_GUARD_SESSION = "guardSession";
     private static final String FIELD_PASSWORD = "password";
     private static final String ENCRYPTED_PASSWORD = "e_password";
     private static final String PGP_SESSION_KEY = "pgp_session_key";
     private static final String FIELD_AUTH = "auth";
-    private static final String JSON = "json";
+    private static final String TYPE = "type";
+
+    private static final String[] KNOWN_TYPES = { "PGP", "SMIME" };  // Known and possibly supported Guard TYPES
 
     /**
      * Initializes a new {@link AuthTokenAction}.
@@ -66,27 +76,7 @@ public class AuthTokenAction extends AbstractGuardAction {
         super(services);
     }
 
-    /**
-     * Internal method to extract a cookie from the incoming request
-     *
-     * @param cookieName The name of the cookie
-     * @param request The request
-     * @return The cookie with the given name, or null if no such cookie was found
-     */
-    private Cookie extractCookieFrom(String cookieName, HttpServletRequest request) {
-        Cookie foundCookie = null;
-        Cookie[] cookies = request.getCookies();
-        if (cookies != null) {
-            for (Cookie cookie : cookies) {
-                if (cookie.getName().startsWith(cookieName)) {
-                    foundCookie = cookie;
-                }
-            }
-        }
-        return foundCookie;
-    }
-
-    private String hash(String value) throws OXException {
+    private static String hash(String value) throws OXException {
         MessageDigest md;
         try {
             md = MessageDigest.getInstance("SHA-512");
@@ -114,14 +104,28 @@ public class AuthTokenAction extends AbstractGuardAction {
      * @throws OXException
      * @throws JSONException
      */
-    private String extractGuardSessionId(AJAXRequestData requestData, ServerSession session) throws OXException, JSONException {
+    private static String extractGuardSessionId(AJAXRequestData requestData, ServerSession session) throws OXException, JSONException {
         JSONObject json = (JSONObject) requestData.getData();
         if (json != null && json.has(FIELD_GUARD_SESSION)) {
             return json.getString(FIELD_GUARD_SESSION);
         }
-        else {
-            return (hash(session.getSecret()));
+        return (hash(session.getSecret()));
+    }
+
+    private static String getType(AJAXRequestData request) throws OXException {
+        if (null != request && request.containsParameter(TYPE)) {
+            final String type = request.getParameter(TYPE);
+            if (type.isEmpty()) {
+                return "PGP";
+            }
+            for (String known : KNOWN_TYPES) {
+                if (known.equals(type.toUpperCase())) {
+                    return type;
+                }
+            }
+            throw GuardExceptionCodes.ACTION_NOT_SUPPORTED.create(request.getParameter(TYPE));
         }
+        return "PGP";  // default
     }
 
     /**
@@ -130,7 +134,7 @@ public class AuthTokenAction extends AbstractGuardAction {
      * @param requestData The request data
      * @return The parsed minutes, or <code>null</code> if the given requests data does not contain a minutesValid value.
      */
-    private Integer extractMinutesValid(AJAXRequestData requestData) {
+    private static Integer extractMinutesValid(AJAXRequestData requestData) {
         String minutesValid = requestData.getParameter(FIELD_MINUTES_VALID);
         if (minutesValid == null) {
             JSONObject json = (JSONObject) requestData.getData();
@@ -138,6 +142,7 @@ public class AuthTokenAction extends AbstractGuardAction {
                 try {
                     minutesValid = json.getString("minutesValid");
                 } catch (JSONException e) {
+                    LOGGER.debug("Unable to get minutes from JSON", e);
                     return null;
                 }
             }
@@ -161,7 +166,7 @@ public class AuthTokenAction extends AbstractGuardAction {
      * @throws JSONException
      * @throws OXException
      */
-    private GuardAuthenticationToken toAuthenticationToken(AJAXRequestData requestData, ServerSession session) throws JSONException, OXException {
+    private static AJAXRequestResult toAuthenticationToken(AJAXRequestData requestData, ServerSession session, CryptoType.PROTOCOL type) throws JSONException, OXException {
         JSONObject json = (JSONObject) requestData.requireData();
 
         if (!json.has(FIELD_AUTH)) {
@@ -169,6 +174,12 @@ public class AuthTokenAction extends AbstractGuardAction {
         }
         String authToken = json.getString(FIELD_AUTH);
 
+        if (authToken.contains("!")) {
+            String typeString = authToken.substring(0, authToken.indexOf("!"));
+            type = CryptoType.getTypeFromString(typeString);
+            authToken = authToken.substring(authToken.indexOf("!") + 1);
+        }
+
         String guardSessionId = extractGuardSessionId(requestData, session);
         if (guardSessionId == null) {
             throw AjaxExceptionCodes.MISSING_FIELD.create(FIELD_GUARD_SESSION);
@@ -178,7 +189,8 @@ public class AuthTokenAction extends AbstractGuardAction {
         if (json.has(FIELD_MINUTES_VALID)) {
             minutesValid = json.getInt(FIELD_MINUTES_VALID);
         }
-        return new GuardAuthenticationToken(guardSessionId, authToken, minutesValid);
+        GuardAuthenticationToken authenticationToken = new GuardAuthenticationToken(guardSessionId, authToken, minutesValid);
+        return doPerformSet(authenticationToken, session, type);
     }
 
     /**
@@ -189,15 +201,21 @@ public class AuthTokenAction extends AbstractGuardAction {
      * @return the token as JSONObject
      * @throws JSONException
      */
-    private JSONObject asJsonResult(GuardAuthenticationToken token, boolean attachedToSession) throws JSONException {
-        if (token != null) {
+    private static JSONArray asJsonResult(List<GuardAuthenticationToken> tokens, boolean attachedToSession) {
+        JSONArray results = new JSONArray();
+        tokens.forEach((token) -> {
             JSONObject result = new JSONObject();
-            result.put("auth", token.getValue());
-            result.put("createdOn", token.getCreatedOn().getTime());
-            result.put("minutesValid", attachedToSession ? I(token.getMinutesValid()) : JSONObject.NULL);
-            return result;
-        }
-        return null;
+            try {
+                result.put("auth", token.getValue());
+                result.put("createdOn", token.getCreatedOn().getTime());
+                result.put("minutesValid", attachedToSession ? I(token.getMinutesValid()) : JSONObject.NULL);
+                result.put("type", token.getType());
+                results.put(result);
+            } catch (JSONException e) {
+                LOGGER.debug("Unable to generate JSON", e);
+            }
+        });
+        return results;
     }
 
     /**
@@ -231,25 +249,24 @@ public class AuthTokenAction extends AbstractGuardAction {
         if (guardSessionId == null) {
             throw AjaxExceptionCodes.MISSING_FIELD.create(FIELD_GUARD_SESSION);
         }
-
-        final String userAgent = requestData.optHttpServletRequest().getHeader("User-Agent");
+        HttpServletRequest request = requestData.optHttpServletRequest();
+        final String userAgent = null == request ? null : request.getHeader("User-Agent");
         if(pgpSessionKey == null) {
             return new GuardLoginHandler().login(
                 session,
                 guardSessionId,
                 password,
                 ePassword,
-                GuardApis.extractCookiesFrom(requestData.optHttpServletRequest(), session),
-                userAgent);
-        }
-        else {
-            return new GuardLoginHandler().loginWithSymmetricKey(
-                session,
-                guardSessionId,
-                pgpSessionKey,
-                GuardApis.extractCookiesFrom(requestData.optHttpServletRequest(), session),
-                userAgent);
+                GuardApis.extractCookiesFrom(request, session),
+                userAgent,
+                getType(requestData));
         }
+        return new GuardLoginHandler().loginWithSymmetricKey(
+            session,
+            guardSessionId,
+            pgpSessionKey,
+            GuardApis.extractCookiesFrom(request, session),
+            userAgent);
     }
 
     /**
@@ -258,14 +275,12 @@ public class AuthTokenAction extends AbstractGuardAction {
      * @param requestData The request data
      * @param session The session
      * @return The result describing whether an authentication token has been attached to a session or not
-     * @throws OXException
-     * @throws JSONException
      */
-    private AJAXRequestResult doPerformGet(AJAXRequestData requestData, ServerSession session) throws OXException, JSONException {
-        GuardAuthenticationToken tokenForSession = new AuthenticationTokenHandler().getForSession(session, false);
+    private static AJAXRequestResult doPerformGet(AJAXRequestData requestData, ServerSession session) {
+        List<GuardAuthenticationToken> tokensForSession = new AuthenticationTokenHandler().getAllForSession(session);
         //  Check for same session Secret?
         requestData.setResponseHeader("content-type", "application/json; charset=UTF-8");
-        return new AJAXRequestResult(asJsonResult(tokenForSession, true), "json");
+        return new AJAXRequestResult(asJsonResult(tokensForSession, true), "json");
     }
 
     /**
@@ -277,17 +292,15 @@ public class AuthTokenAction extends AbstractGuardAction {
      * @throws OXException
      * @throws JSONException
      */
-    private AJAXRequestResult doPerformSet(GuardAuthenticationToken authenticationToken, ServerSession session) throws OXException, JSONException {
+    private static AJAXRequestResult doPerformSet(GuardAuthenticationToken authenticationToken, ServerSession session, CryptoType.PROTOCOL type) throws OXException {
         if (authenticationToken.getValue() == null || authenticationToken.getValue().isEmpty()) {
             //Detach existing token
-            new AuthenticationTokenHandler().destroyForSession(session, false);
+            new AuthenticationTokenHandler().destroyForSession(session, false, type.getValue());
             return new AJAXRequestResult(null, "json");
         }
-        else {
-            //Attach a new token
-            new AuthenticationTokenHandler().setForSession(session, authenticationToken);
-            return new AJAXRequestResult(asJsonResult(authenticationToken, true), "json");
-        }
+        //Attach a new token
+        new AuthenticationTokenHandler().setForSession(session, authenticationToken, type.getValue());
+        return new AJAXRequestResult(asJsonResult(Collections.singletonList(authenticationToken), true), "json");
     }
 
     /*
@@ -297,11 +310,13 @@ public class AuthTokenAction extends AbstractGuardAction {
      */
     @Override
     protected AJAXRequestResult doPerform(AJAXRequestData requestData, ServerSession session) throws OXException, JSONException {
-        if (requestData.optHttpServletRequest().getMethod().equals("GET")) {
+        CryptoType.PROTOCOL type = CryptoType.getTypeFromString(requestData.getParameter("type"));
+        HttpServletRequest request = requestData.optHttpServletRequest();
+        if (null != request && request.getMethod().equals("GET")) {
             //GET -> Query existing token
             return doPerformGet(requestData, session);
         }
-        else if(requestData.optHttpServletRequest().getMethod().equals("POST")) {
+        else if(null != request && request.getMethod().equals("POST")) {
             //POST -> Perform direct authentication against Guard and attach returned token if valid
             GuardAuthenticationToken token = doAuthentication(requestData, session);
             if(token != null) {
@@ -309,19 +324,16 @@ public class AuthTokenAction extends AbstractGuardAction {
                 Integer minutesValid = extractMinutesValid(requestData);
                 token.setMinutesValid(minutesValid != null ? i(minutesValid) : -1);
                 if(minutesValid != null) {
-                    return doPerformSet(token, session);
+                    return doPerformSet(token, session, type);
                 }
-                return new AJAXRequestResult(asJsonResult(token, false), "json");
-            }
-            else {
-                //Authentication failed
-                throw GuardExceptionCodes.BAD_AUTH.create();
+                return new AJAXRequestResult(asJsonResult(Collections.singletonList(token), false), "json");
             }
+            //Authentication failed
+            throw GuardExceptionCodes.BAD_AUTH.create();
         }
         else {
             //PUT -> Attach/Detach existing authentication token sent by client
-            GuardAuthenticationToken authenticationToken = toAuthenticationToken(requestData, session);
-            return doPerformSet(authenticationToken, session);
+            return toAuthenticationToken(requestData, session, type);
         }
     }
 }
