@@ -21,11 +21,11 @@
 
 package com.openexchange.guard.mail.service.impl;
 
+import com.openexchange.crypto.CryptoType;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.exceptions.GuardExceptionCodes;
 import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
 import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
-import com.openexchange.java.Reference;
 import com.openexchange.java.Strings;
 import com.openexchange.mail.IndexRange;
 import com.openexchange.mail.MailExceptionCode;
@@ -43,12 +43,15 @@ import com.openexchange.mail.api.ISimplifiedThreadStructureEnhanced;
 import com.openexchange.mail.api.MailMessageStorage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.dataobjects.MailPart;
+import com.openexchange.mail.dataobjects.SecurityInfo;
 import com.openexchange.mail.dataobjects.compose.ComposedMailMessage;
+import com.openexchange.mail.mime.crypto.CryptoMailRecognizerService;
 import com.openexchange.mail.mime.crypto.PGPMailRecognizer;
 import com.openexchange.mail.parser.MailMessageParser;
 import com.openexchange.mail.parser.handlers.ImageMessageHandler;
 import com.openexchange.mail.parser.handlers.MailPartHandler;
 import com.openexchange.mail.search.SearchTerm;
+import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 
 /**
@@ -64,8 +67,9 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
 
     protected final IMailMessageStorage delegate;
     protected final Session session;
-    protected final PGPMailRecognizer pgpMailRecognizer;
+    protected final ServiceLookup services;
     private final GuardAuthenticationToken authToken;
+    private final CryptoType.PROTOCOL type;
 
     /**
      * Initializes a new {@link GuardMessageStorage}.
@@ -74,11 +78,12 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
      * @param delegate The underlying {@link IMailMessageStorage} to wrap with guard.
      * @param pgpMailRecognizer A {@link PGPMailRecognizer} object recognizes PGP messages.
      */
-    public GuardMessageStorage(Session session, IMailMessageStorage delegate, PGPMailRecognizer pgpMailRecognizer, GuardAuthenticationToken authToken) {
+    public GuardMessageStorage(Session session, IMailMessageStorage delegate, ServiceLookup services, GuardAuthenticationToken authToken, CryptoType.PROTOCOL type) {
         this.session = session;
         this.delegate = delegate;
-        this.pgpMailRecognizer = pgpMailRecognizer;
+        this.services = services;
         this.authToken = authToken;
+        this.type = type;
     }
 
     @Override
@@ -89,32 +94,32 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
         }
 
         if (IMailMessageStorageExt.class == iface) {
-            return (T) new GuardMessageStorageExt(session, (IMailMessageStorageExt) instance, pgpMailRecognizer, authToken);
+            return (T) new GuardMessageStorageExt(session, (IMailMessageStorageExt) instance, services, authToken, type);
         }
 
         if (IMailMessageStorageMimeSupport.class == iface) {
-            return (T) new GuardMessageStorageMimeSupport(session, (IMailMessageStorageMimeSupport) instance, pgpMailRecognizer, authToken);
+            return (T) new GuardMessageStorageMimeSupport(session, (IMailMessageStorageMimeSupport) instance, services, authToken, type);
         }
 
         if (IMailMessageStorageThreadReferences.class == iface) {
-            return (T) new GuardMessageStorageThreadReferences(session, (IMailMessageStorageThreadReferences) instance, pgpMailRecognizer, authToken);
+            return (T) new GuardMessageStorageThreadReferences(session, (IMailMessageStorageThreadReferences) instance, services, authToken, type);
         }
 
         if (ISimplifiedThreadStructureEnhanced.class == iface) {
-            return (T) new GuardMessageStorageThreadStructureEnhanced(session, (ISimplifiedThreadStructureEnhanced) instance, pgpMailRecognizer, authToken);
+            return (T) new GuardMessageStorageThreadStructureEnhanced(session, (ISimplifiedThreadStructureEnhanced) instance, services, authToken, type);
         }
 
         if (ISimplifiedThreadStructure.class == iface) {
-            return (T) new GuardMessageStorageThreadStructure(session, (ISimplifiedThreadStructure) instance, pgpMailRecognizer, authToken);
+            return (T) new GuardMessageStorageThreadStructure(session, (ISimplifiedThreadStructure) instance, services, authToken, type);
         }
 
         return instance;
     }
 
-    protected GuardAuthenticationToken getAuthToken(Session session) throws OXException {
+    protected GuardAuthenticationToken getAuthToken(Session session, CryptoType.PROTOCOL type, boolean markUsed) throws OXException {
         return this.authToken != null ?
-            this.authToken :
-            new AuthenticationTokenHandler().getForSession(session);
+            this.authToken.getToken(type) :
+            new AuthenticationTokenHandler().getForSession(session, markUsed, type.getValue());
     }
 
     /**
@@ -124,8 +129,9 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
      * @return True, if the message is a PGP message, false otherwise
      * @throws OXException
      */
-    private boolean isPGPMessage(MailMessage message) throws OXException {
-        return pgpMailRecognizer.isPGPMessage(message);
+    private boolean isCryptoMessage(MailMessage message) throws OXException {
+        CryptoMailRecognizerService cryptoMailRecognizer = services.getServiceSafe(CryptoMailRecognizerService.class);
+        return cryptoMailRecognizer.isCryptoMessage(message);
     }
 
     /**
@@ -135,8 +141,14 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
      * @return
      * @throws OXException
      */
-    private boolean isPGPSignedMessage(MailMessage message) throws OXException {
-        return pgpMailRecognizer.isPGPSignedMessage(message);
+    private boolean isSignedMessage(MailMessage message) throws OXException {
+        CryptoMailRecognizerService cryptoMailRecognizer = services.getServiceSafe(CryptoMailRecognizerService.class);
+        return cryptoMailRecognizer.isSignedMessage(message);
+    }
+
+    private CryptoType.PROTOCOL getTypeCrypto(MailMessage message) throws OXException {
+        CryptoMailRecognizerService cryptoMailRecognizer = services.getServiceSafe(CryptoMailRecognizerService.class);
+        return cryptoMailRecognizer.getTypeCrypto(message);
     }
 
     /**
@@ -160,7 +172,14 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
      * @throws OXException
      */
     protected MailMessage optProcessMessage(MailMessage mailMessage) throws OXException {
-        GuardAuthenticationToken authToken = getAuthToken(session);
+        SecurityInfo info = mailMessage.getSecurityInfo();
+        CryptoType.PROTOCOL type = info == null ? getTypeCrypto(mailMessage) : info.getType();
+        if (type == null) {
+            LOG.error("Unknown crypto type");
+            return mailMessage;
+        }
+        GuardAuthenticationToken authToken = getAuthToken(session, type, false);  // Grab token non-destructively
+
         final GuardMimeDecrypter decrypter = new GuardMimeDecrypter();
 
         MailMessage mail = mailMessage;
@@ -174,18 +193,18 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
         Boolean hasAttachment = mail.containsHasAttachment() ? Boolean.valueOf(mail.isHasAttachment()) : null;
 
         boolean changed = false;
-        if (isPGPMessage(mail)) {
+        if (isCryptoMessage(mail)) {
             if (authToken == null) {
                 LOG.error("Missing / Bad authentication");
                 throw GuardExceptionCodes.MISSING_AUTH.create();
             }
 
-            mail = decrypter.decrypt(session, authToken, mail);
+            mail = decrypter.decrypt(session, authToken, mail, getTypeCrypto(mail));
             changed = true;
         }
-        if (isPGPSignedMessage(mail)) {
+        if (isSignedMessage(mail)) {
             GuardMimeVerifier verifier = new GuardMimeVerifier();
-            mail = verifier.verify(session, mail);
+            mail = verifier.verify(session, mail, type);
             changed = true;
         }
         if (false == changed) {
@@ -193,6 +212,8 @@ public class GuardMessageStorage extends MailMessageStorage implements IMailMess
             return null;
         }
 
+        getAuthToken(session, type, true);  // Mark token as used now.
+
         // Restore flags to decoded email
         if (flags >= 0) {
             mail.setFlags(flags);
